class Session:
    def __init__(self, libtorrent, port='6881'):
        self._user_agent = 'python client v0.1'
        self._listen_interfaces = '0.0.0.0'
        self._port = port
        self._download_rate_limit = 0
        self._upload_rate_limit = 0
        self._lt = libtorrent
        self._session = None

    def create_session(self):
        self._session = self._lt.session({'listen_interfaces':f'{self._listen_interfaces}:{self._port}'})
        return self._session

    def set_download_limit(self, rate=0):
        self._download_rate_limit = int(-1 if rate == 0 else (1 if rate == -1 else rate * 1024))
        self._session.set_download_rate_limit(self._download_rate_limit)

    def set_upload_limit(self, rate=0):
        self._upload_rate_limit = int(-1 if rate == 0 else (1 if rate == -1 else rate * 1024))
        self._session.set_upload_rate_limit(self._upload_rate_limit)

    def get_upload_limit(self):
        return self._session.upload_rate_limit()

    def get_download_limit(self):
        return self._session.download_rate_limit()

    def __str__(self):
        pass

    def __repr__(self):
        pass

    def __call__(self):
        return self.create_session()



import os.path
import pathlib
import re

from setuptools import setup

PROJECT_NAME = 'pytorrentdl'
# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()


def get_property(prop):
    result = re.search(r'{}\s*=\s*[\'"]([^\'"]*)[\'"]'.format(prop),
                       open(os.path.join(PROJECT_NAME, '__init__.py')).read())
    return result.group(1)


setup(
    name='pytorrentdl',
    version=get_property('__version__'),
    description='Download from torrent with magnet link or .torrent file',
    long_description=README,
    long_description_content_type="text/markdown",
    url=get_property('__url__'),
    author=get_property('__author__'),
    author_email=get_property('__author_email__'),
    license=get_property('__license__'),
    packages=['pytorrentdl'],
    entry_points={
        'console_scripts': ['pytorrentdl=pytorrentdl.cli:run_cli']
    },
    install_requires=['libtorrent>=2.0.7', 'asyncclick>=8.1.7.2', ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: POSIX :: Linux',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],
)



from .torrent_downloader import TorrentDownloader
import asyncclick as click
import asyncio

async def handle_input(torrent_file):
    while True:
        action = await asyncio.get_event_loop().run_in_executor(None, input, "Enter action (pause/resume/stop): ")
        if action.lower() == 'pause':
            torrent_file.pause_download()
        elif action.lower() == 'resume':
            torrent_file.resume_download()
        elif action.lower() == 'stop':
            torrent_file.stop_download()
            print("The Program is exiting...")
            raise SystemExit
        else:
            print("Invalid action. Please enter 'pause', 'resume', or 'stop'.")

@click.command()
@click.option('--link', required=True, help="Torrent link. Example: [--link 'file.torrent'] or [--link 'magnet:...']", type=str)
@click.option('--download_speed',  default=0, help='Download speed with a specific number (kB/s). Default: 0, means unlimited speed', type=int)
@click.option('--upload_speed',  default=0, help='Upload speed with a specific number (kB/s). Default: 0, means unlimited speed', type=int)
@click.option('--save_path',  default='.', help="Path to save the file, default: '.' ", type=str)
async def run_cli(link, download_speed, upload_speed, save_path):
    try:
        torrent_file = TorrentDownloader(link, save_path)
        
        input_task = asyncio.create_task(handle_input(torrent_file))
        download_task = asyncio.create_task(torrent_file.start_download(download_speed=download_speed, upload_speed=upload_speed))
        
        await asyncio.gather(input_task, download_task)

    except asyncio.TimeoutError or KeyboardInterrupt:
        print("The Program is terminated manually!")
        raise SystemExit


if __name__ == '__main__':
    run_cli(_anyio_backend="asyncio")
